@@ -2,7 +2,6 @@ package File::DataClass::IO;
 
 use 5.010001;
 
-use Moo;
 use Cwd                        qw( );
 use English                    qw( -no_match_vars );
 use Exporter 5.57              qw( import );
@@ -25,6 +24,7 @@ use Type::Utils                qw( enum );
 use Unexpected::Functions      qw( InvocantUndefined PathNotFound Unspecified );
 use Unexpected::Types          qw( ArrayRef Bool CodeRef Int Maybe Object
                                    PositiveInt RegexpRef SimpleStr Str );
+use Moo;
 
 use namespace::clean -except => [ 'import', 'meta' ];
 use overload '""'       => sub { $_[ 0 ]->as_string  },
@@ -52,13 +52,13 @@ my $_expand_tilde = sub {
 my $_coerce_name = sub {
    my $name = shift;
 
-   not defined $name          and return;
-   is_coderef  $name          and $name =  $name->();
-   blessed     $name          and $name =  "${name}";
-   is_arrayref $name          and $name =  File::Spec->catfile( @{ $name } );
-   curdir eq   $name          and $name =  Cwd::getcwd();
-   first_char  $name eq TILDE and $name =  $_expand_tilde->( $name );
-   length      $name > 1      and $name =~ s{ [/\\] \z }{}mx;
+   not defined  $name          and return;
+   is_coderef   $name          and $name =  $name->();
+   blessed      $name          and $name =  "${name}";
+   is_arrayref  $name          and $name =  File::Spec->catfile( @{ $name } );
+   curdir eq    $name          and $name =  Cwd::getcwd();
+   first_char   $name eq TILDE and $name =  $_expand_tilde->( $name );
+   CORE::length $name > 1      and $name =~ s{ [/\\] \z }{}mx;
    return $name;
 };
 
@@ -321,7 +321,7 @@ my $_assert_open_backwards = sub {
 my $_init_type_from_fs = sub {
    my $self = shift;
 
-   $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
+   CORE::length $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
 
    return -f $self->name ? $self->file : -d _ ? $self->dir : undef;
 };
@@ -329,7 +329,7 @@ my $_init_type_from_fs = sub {
 my $_open_args = sub {
    my ($self, $mode, $perms) = @_;
 
-   $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
+   CORE::length $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
 
    my $pathname = $self->_atomic && !$self->is_reading( $mode )
                 ? $self->$_get_atomic_path : $self->name;
@@ -451,7 +451,8 @@ sub abs2rel {
 sub absolute {
    my ($self, $base) = @_; $base and $base = $_coerce_name->( $base );
 
-   $self->_set_name( $self->name ? $self->rel2abs( $base ) : $base );
+   $self->_set_name
+      ( (CORE::length $self->name) ? $self->rel2abs( $base ) : $base );
    return $self;
 }
 
@@ -490,17 +491,27 @@ sub appendln {
 }
 
 sub as_boolean {
-   return ($_[ 0 ]->name || $_[ 0 ]->io_handle) ? TRUE : FALSE;
+   return ((CORE::length $_[ 0 ]->name) || $_[ 0 ]->io_handle) ? TRUE : FALSE;
 }
 
 sub as_string {
-   my $self = shift;
+   my $self = shift; CORE::length $self->name and return $self->name;
 
-   return $self->name || (defined $self->io_handle && NUL.$self->io_handle);
+   defined $self->io_handle and return NUL.$self->io_handle;
+
+   return NUL;
 }
 
 sub assert {
-   $_[ 0 ]->_assert( TRUE ); return $_[ 0 ];
+   my ($self, $cb) = @_;
+
+   if ($cb) {
+      local $_ = $self;
+      $cb->() or throw 'Path [_1] assertion failure', [ $self->name ];
+   }
+   else { $self->_assert( TRUE ) }
+
+   return $self;
 }
 
 sub assert_dirpath {
@@ -526,7 +537,7 @@ sub assert_dirpath {
 sub assert_filepath {
    my $self = shift; my $dir;
 
-   $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
+   CORE::length $self->name or $self->$_throw( Unspecified, [ 'path name' ] );
 
    (undef, $dir) = File::Spec->splitpath( $self->name );
 
@@ -555,7 +566,7 @@ sub backwards {
 }
 
 sub basename {
-   my ($self, @suffixes) = @_; $self->name or return;
+   my ($self, @suffixes) = @_; CORE::length $self->name or return;
 
    return File::Basename::basename( $self->name, @suffixes );
 }
@@ -602,21 +613,20 @@ sub canonpath {
 }
 
 sub catdir {
-   my ($self, @rest) = @_;
-
-   my $params = (is_hashref $rest[ -1 ]) ? pop @rest : {};
-   my $args   = [ grep { defined and length } $self->name, @rest ];
-
-   return $self->$_constructor( $args, $params )->dir;
+   my ($self, @args) = @_; return $self->child( @args )->dir;
 }
 
 sub catfile {
-   my ($self, @rest) = @_;
+   my ($self, @args) = @_; return $self->child( @args )->file;
+}
 
-   my $params = (is_hashref $rest[ -1 ]) ? pop @rest : {};
-   my $args   = [ grep { defined and length } $self->name, @rest ];
+sub child {
+   my ($self, @args) = @_;
 
-   return $self->$_constructor( $args, $params )->file;
+   my $params = (is_hashref $args[ -1 ]) ? pop @args : {};
+   my $args   = [ grep { defined and CORE::length } $self->name, @args ];
+
+   return $self->$_constructor( $args, $params );
 }
 
 sub chmod {
@@ -732,7 +742,8 @@ sub dir {
 }
 
 sub dirname {
-   return $_[ 0 ]->name ? File::Basename::dirname( $_[ 0 ]->name ) : undef;
+   return CORE::length $_[ 0 ]->name ? File::Basename::dirname( $_[ 0 ]->name )
+                                     : NUL;
 }
 
 sub encoding {
@@ -755,7 +766,7 @@ sub error_check {
 }
 
 sub exists {
-   return ($_[ 0 ]->name && -e $_[ 0 ]->name) ? TRUE : FALSE;
+   return (CORE::length $_[ 0 ]->name && -e $_[ 0 ]->name) ? TRUE : FALSE;
 }
 
 sub file {
@@ -837,7 +848,7 @@ sub is_absolute {
 }
 
 sub is_dir {
-   my $self = shift; $self->name or return FALSE;
+   my $self = shift; CORE::length $self->name or return FALSE;
 
    $self->type or $self->$_init_type_from_fs or return FALSE;
 
@@ -856,11 +867,11 @@ sub is_empty {
 *empty = \&is_empty; # Deprecated
 
 sub is_executable {
-   return $_[ 0 ]->name && -x $_[ 0 ]->name ? TRUE : FALSE;
+   return (CORE::length $_[ 0 ]->name) && -x $_[ 0 ]->name ? TRUE : FALSE;
 }
 
 sub is_file {
-   my $self = shift; $self->name or return FALSE;
+   my $self = shift; CORE::length $self->name or return FALSE;
 
    $self->type or $self->$_init_type_from_fs;
 
@@ -868,11 +879,11 @@ sub is_file {
 }
 
 sub is_link {
-   return $_[ 0 ]->name && -l $_[ 0 ]->name ? TRUE : FALSE;
+   return (CORE::length $_[ 0 ]->name) && -l $_[ 0 ]->name ? TRUE : FALSE;
 }
 
 sub is_readable {
-   return $_[ 0 ]->name && -r $_[ 0 ]->name ? TRUE : FALSE;
+   return (CORE::length $_[ 0 ]->name) && -r $_[ 0 ]->name ? TRUE : FALSE;
 }
 
 sub is_reading {
@@ -880,7 +891,7 @@ sub is_reading {
 }
 
 sub is_writable {
-   return $_[ 0 ]->name && -w $_[ 0 ]->name ? TRUE : FALSE;
+   return (CORE::length $_[ 0 ]->name) && -w $_[ 0 ]->name ? TRUE : FALSE;
 }
 
 sub is_writing {
@@ -890,9 +901,12 @@ sub is_writing {
 }
 
 sub iterator {
-   my $self = shift; my $deep = $self->_deep; my @dirs = ( $self );
+   my ($self, $args) = @_;
 
-   my $filter = $self->_filter; my $follow = not $self->_no_follow;
+   my @dirs   = ( $self );
+   my $filter = $self->_filter;
+   my $deep   = $args->{recurse} // $self->_deep;
+   my $follow = $args->{follow_symlinks} // not $self->_no_follow;
 
    return sub {
       while (@dirs) {
@@ -910,7 +924,7 @@ sub iterator {
 }
 
 sub length {
-   return length ${ $_[ 0 ]->buffer };
+   return CORE::length ${ $_[ 0 ]->buffer };
 }
 
 sub lock {
@@ -1121,6 +1135,10 @@ sub set_lock {
    return $self;
 }
 
+sub sibling {
+   my $self = shift; return $self->parent->child( @_ );
+}
+
 sub slurp {
    my $self = shift; my $slurp = $self->all;
 
@@ -1199,7 +1217,7 @@ sub tempfile {
 }
 
 sub touch {
-   my ($self, $time) = @_; $self->name or return; $time //= time;
+   my ($self, $time) = @_; CORE::length $self->name or return; $time //= time;
 
    -e $self->name or $self->$_open_file( $self->$_open_args( 'w' ) )->close;
 
@@ -1223,6 +1241,20 @@ sub utf8 {
    $_[ 0 ]->encoding( 'UTF-8' ); return $_[ 0 ];
 }
 
+sub visit {
+   my ($self, $cb, $args) = @_;
+
+   my $iter = $self->iterator( $args ); my $state = {};
+
+   while (defined (my $entry = $iter->())) {
+      local $_ = $entry; my $r = $cb->( $entry, $state );
+
+      ref $r and not ${ $r } and last;
+   }
+
+   return $state;
+}
+
 sub write {
    my ($self, @args) = @_; $self->assert_open( 'w' );
 
@@ -1443,6 +1475,12 @@ Sets the private attribute C<_assert> to true. Causes the open methods
 to create the path to the directory before the file/directory is
 opened
 
+   $io = io( 'path_to_file' )->assert( sub { $_->exists } );
+
+When called with a code reference it sets C<$_> to self and asserts that
+the code reference returns true. Throws otherwise. This feature was copied
+from L<Path::Tiny>
+
 =head2 assert_dirpath
 
    $dir_name = io( 'path_to_file' )->assert_dirpath;
@@ -1555,6 +1593,12 @@ with the one that is supplied
 Create a new IO file object by concatenating this objects pathname
 with the one that is supplied
 
+=head2 child
+
+   $io = io( 'path_to_directory' )->child( 'additional_file_path' );
+
+Like L</catdir> and L</catfile> but does not set the object type
+
 =head2 chmod
 
    $io = io( 'path_to_file' )->chmod( '0644' );
@@ -1828,12 +1872,13 @@ Returns true if this IO object is in one of the write modes
 
 =head2 iterator
 
-   $code_ref = io( 'path_to_directory' )->iterator;
+   $code_ref = io( 'path_to_directory' )->iterator( $options );
 
-When called the coderef iterates over the directory listing. If C<deep> is
-true then the iterator will visit all subdirectories. If C<no_follow> is
-true then symbolic links to directories will no be followed. A L</filter>
-may also be applied
+When called the coderef iterates over the directory listing. If C<deep> is true
+then the iterator will visit all subdirectories. If C<no_follow> is true then
+symbolic links to directories will not be followed. A L</filter> may also be
+applied. The options hash takes C<recurse> which is used in preference to
+C<deep>, and C<follow_symlinks> should be defined or C<no_follow> will be used
 
 =head2 length
 
@@ -2011,6 +2056,13 @@ Sets the currently selected binmode on the open file handle
 
 Calls L</flock> on the open file handle
 
+=head2 sibling
+
+   $io = io( 'path_to_directory' )->sibling( 'additional_relative_path' );
+
+A shortcut for calling C<< $io->parent->child >>. This feature was copied
+from L<Path::Tiny>
+
 =head2 slurp
 
    $lines = io( 'path_to_file' )->slurp;
@@ -2122,6 +2174,20 @@ release the L<Fcntl> lock if one was set. Called by the L</close> method
 
 Sets the current encoding to utf8
 
+=head2 visit
+
+   $state = io( 'path_to_directory' )->visit( \&callback, $options );
+
+Wrapper around a call to L</iterator>, calls the callback subroutine for
+each entry. The options hash takes C<recurse> to set L</deep> to true and
+C<follow_symlinks> should be true or L</no_follow> will be called.  The
+callback subroutine is passed the io object reference and a hash reference
+in which to accumulate state. In the callback subroutine C<$_> is also
+localised to the current entry. The state hash reference is returned by
+the method call. If the callback subroutine return a reference to a false
+scalar value the loop around the call to L</iterator> terminates and the
+state hash reference is returned. This feature was copied from L<Path::Tiny>
+
 =head2 write
 
    $bytes_written = io( 'pathname' )->write( $buffer, $length );
@@ -2187,7 +2253,7 @@ For L<IO::All> from which I took the API and some tests
 =item L<Path::Tiny>
 
 Lifted the following features; iterator, tilde expansion, thread id in atomic
-file name, not following symlinks and some tests
+file name, not following symlinks, visit, sibling and some tests
 
 =back
 
