@@ -2,7 +2,6 @@ package File::DataClass::ResultSet;
 
 use namespace::autoclean;
 
-use Moo;
 use File::DataClass::Constants qw( EXCEPTION_CLASS FALSE TRUE );
 use File::DataClass::Functions qw( is_arrayref is_hashref is_member throw );
 use File::DataClass::List;
@@ -12,6 +11,7 @@ use File::DataClass::Types     qw( ArrayRef ClassName
 use Scalar::Util               qw( blessed );
 use Subclass::Of;
 use Unexpected::Functions      qw( RecordNotFound Unspecified );
+use Moo;
 
 my $class_stash = {};
 
@@ -294,7 +294,7 @@ sub delete {
       }
 
       $result->delete
-         or throw 'File [_1] element [_2] not deleted', [ $path, $id ];
+         or throw 'File [_1] source [_2] not deleted', [ $path, $id ];
       return TRUE;
    } );
 
@@ -412,21 +412,21 @@ __END__
 
 =head1 Name
 
-File::DataClass::ResultSet - Core element methods
+File::DataClass::ResultSet - Core result management methods
 
 =head1 Synopsis
 
    use File:DataClass;
 
-   $attrs = { result_source_attributes => { schema_attributes => { ... } } };
+   my $attr = { result_source_attributes => { schema_attributes => { ... } } };
 
-   $result_source = File::DataClass->new( $attrs )->result_source;
+   my $result_source = File::DataClass->new( $attr )->result_source;
 
-   $rs = $result_source->resultset( { path => q(path_to_data_file) } );
+   my $rs = $result_source->resultset( { path => q(path_to_data_file) } );
 
-   $result = $rs->search( $hash_ref_of_where_clauses );
+   my $result = $rs->search( $hash_ref_of_where_clauses );
 
-   for $result_object ($result->next) {
+   for my $result_object ($result->next) {
       # Do something with the result object
    }
 
