@@ -2,29 +2,37 @@ package File::DataClass::Storage;
 
 use namespace::autoclean;
 
-use Moo;
 use Class::Null;
 use English                    qw( -no_match_vars );
 use File::Copy;
 use File::DataClass::Constants qw( EXCEPTION_CLASS FALSE NUL TRUE );
-use File::DataClass::Functions qw( is_stale merge_file_data throw );
-use File::DataClass::HashMerge;
-use File::DataClass::Types     qw( Object Str );
+use File::DataClass::Functions qw( is_stale merge_file_data
+                                   merge_for_update throw );
+use File::DataClass::Types     qw( Bool HashRef Object Str );
 use Scalar::Util               qw( blessed );
 use Try::Tiny;
 use Unexpected::Functions      qw( RecordAlreadyExists PathNotFound
                                    NothingUpdated Unspecified );
+use Moo;
 
-has 'backup'   => is => 'ro', isa => Str, default => NUL;
+has 'atomic_write'  => is => 'ro', isa => Bool, default => TRUE;
 
-has 'encoding' => is => 'ro', isa => Str, default => NUL;
+has 'backup'        => is => 'ro', isa => Str,  default => NUL;
 
-has 'extn'     => is => 'ro', isa => Str, default => NUL;
+has 'encoding'      => is => 'ro', isa => Str,  default => NUL;
 
-has 'schema'   => is => 'ro', isa => Object,
-   handles     => { _cache => 'cache', _lock  => 'lock',
-                    _log   => 'log',   _perms => 'perms', },
-   required    => TRUE,  weak_ref => TRUE;
+has 'extn'          => is => 'ro', isa => Str,  default => NUL;
+
+has 'read_options'  => is => 'ro', isa => HashRef, builder => sub { {} };
+
+has 'schema'        => is => 'ro', isa => Object,
+   handles          => { _cache => 'cache', _lock  => 'lock',
+                         _log   => 'log',   _perms => 'perms', },
+   required         => TRUE,  weak_ref => TRUE;
+
+has 'write_options' => is => 'ro', isa => HashRef, builder => sub { {} };
+
+has '_locks'        => is => 'ro', isa => HashRef, builder => sub { {} };
 
 # Private functions
 my $_get_src_attributes = sub {
@@ -35,30 +43,46 @@ my $_get_src_attributes = sub {
                  and $cond->( $_ ) } keys %{ $src };
 };
 
+my $_lock_set = sub {
+   $_[ 0 ]->_lock->set( k => $_[ 1 ] ); $_[ 0 ]->_locks->{ $_[ 1 ] } = TRUE;
+};
+
+my $_lock_reset = sub {
+   $_[ 0 ]->_lock->reset( k => $_[ 1 ] ); delete $_[ 0 ]->_locks->{ $_[ 1 ] };
+};
+
+my $_lock_reset_all = sub {
+   my $self = shift;
+
+   eval { $self->$_lock_reset( $_ ) } for (keys %{ $self->_locks });
+
+   return;
+};
+
 # Public methods
 sub create_or_update {
    my ($self, $path, $result, $updating, $cond) = @_;
 
-   my $element = $result->result_source->name;
+   my $rsrc_name = $result->result_source->name;
 
-   $self->validate_params( $path, $element ); my $updated;
+   $self->validate_params( $path, $rsrc_name ); my $updated;
 
    my $data = ($self->read_file( $path, TRUE ))[ 0 ];
 
    try {
       my $filter = sub { $_get_src_attributes->( $cond, $_[ 0 ] ) };
-      my $id     = $result->id; $data->{ $element } ||= {};
+      my $id     = $result->id; $data->{ $rsrc_name } //= {};
 
-      not $updating and exists $data->{ $element }->{ $id }
+      not $updating and exists $data->{ $rsrc_name }->{ $id }
          and throw RecordAlreadyExists, [ $path, $id ], level => 2;
 
-      $updated = File::DataClass::HashMerge->merge
-         ( \$data->{ $element }->{ $id }, $result, $filter );
+      $updated = merge_for_update
+         ( \$data->{ $rsrc_name }->{ $id }, $result, $filter );
    }
-   catch { $self->_lock->reset( k => $path ); throw $_ };
+   catch { $self->$_lock_reset( $path ); throw $_ };
 
    if ($updated) { $self->write_file( $path, $data, not $updating ) }
-   else { $self->_lock->reset( k => $path ) }
+   else { $self->$_lock_reset( $path ) }
 
    return $updated ? $result : FALSE;
 }
@@ -66,29 +90,32 @@ sub create_or_update {
 sub delete {
    my ($self, $path, $result) = @_;
 
-   my $element = $result->result_source->name;
+   my $rsrc_name = $result->result_source->name;
 
-   $self->validate_params( $path, $element );
+   $self->validate_params( $path, $rsrc_name );
 
    my $data = ($self->read_file( $path, TRUE ))[ 0 ]; my $id = $result->id;
 
-   if (exists $data->{ $element } and exists $data->{ $element }->{ $id }) {
-      delete $data->{ $element }->{ $id };
-      scalar keys %{ $data->{ $element } } or delete $data->{ $element };
+   if (exists $data->{ $rsrc_name } and exists $data->{ $rsrc_name }->{ $id }) {
+      delete $data->{ $rsrc_name }->{ $id };
+      scalar keys %{ $data->{ $rsrc_name } } or delete $data->{ $rsrc_name };
       $self->write_file( $path, $data );
       return TRUE;
    }
 
-   $self->_lock->reset( k => $path );
+   $self->$_lock_reset( $path );
    return FALSE;
 }
 
+sub DEMOLISH {
+   my $self = shift; $self->$_lock_reset_all(); return;
+}
+
 sub dump {
    my ($self, $path, $data) = @_;
 
    return $self->txn_do( $path, sub {
-      $self->_lock->set( k => $path );
-      $self->write_file( $path, $data, TRUE ) } );
+      $self->$_lock_set( $path ); $self->write_file( $path, $data, TRUE ) } );
 }
 
 sub insert {
@@ -112,8 +139,8 @@ sub load {
    for my $path (@paths) {
       my ($red, $path_mtime) = $self->read_file( $path, FALSE );
 
-      merge_file_data $loaded, $red;
       $path_mtime > $newest and $newest = $path_mtime;
+      merge_file_data $loaded, $red;
    }
 
    $self->_cache->set_by_paths( \@paths, $loaded, $self->meta_pack( $newest ) );
@@ -131,7 +158,7 @@ sub meta_unpack { # Modified in a subclass
 sub read_file {
    my ($self, $path, $for_update) = @_;
 
-   $self->_lock->set( k => $path ); my ($data, $path_mtime);
+   $self->$_lock_set( $path ); my ($data, $path_mtime);
 
    try {
       my $stat = $path->stat; defined $stat and $path_mtime = $stat->{mtime};
@@ -151,9 +178,9 @@ sub read_file {
       }
       else { $self->_log->debug( "Read cache ${path}" ) }
    }
-   catch { $self->_lock->reset( k => $path ); throw $_ };
+   catch { $self->$_lock_reset( $path ); throw $_ };
 
-   $for_update or $self->_lock->reset( k => $path );
+   $for_update or $self->$_lock_reset( $path );
 
    return ($data, $path_mtime);
 }
@@ -164,29 +191,29 @@ sub read_from_file {
 }
 
 sub select {
-   my ($self, $path, $element) = @_;
+   my ($self, $path, $rsrc_name) = @_;
 
-   $self->validate_params( $path, $element );
+   $self->validate_params( $path, $rsrc_name );
 
    my $data = ($self->read_file( $path, FALSE ))[ 0 ];
 
-   return exists $data->{ $element } ? $data->{ $element } : {};
+   return exists $data->{ $rsrc_name } ? $data->{ $rsrc_name } : {};
 }
 
 sub txn_do {
-   my ($self, $path, $code_ref) = @_; my $wantarray = wantarray;
+   my ($self, $path, $code_ref) = @_;
 
-   $self->validate_params( $path, TRUE ); my $key = "txn:${path}";
+   my $wantarray = wantarray; $self->validate_params( $path, TRUE );
 
-   $self->_lock->set( k => $key ); my $res;
+   my $key = "txn:${path}"; $self->$_lock_set( $key ); my $res;
 
    try {
       if ($wantarray) { $res = [ $code_ref->() ] }
       else { $res = $code_ref->() }
    }
-   catch { $self->_lock->reset( k => $key ); throw $_, { level => 4 } };
+   catch { $self->$_lock_reset( $key ); throw $_, { level => 4 } };
 
-   $self->_lock->reset( k => $key );
+   $self->$_lock_reset( $key );
 
    return $wantarray ? @{ $res } : $res;
 }
@@ -203,11 +230,11 @@ sub update {
 }
 
 sub validate_params {
-   my ($self, $path, $element) = @_;
+   my ($self, $path, $rsrc_name) = @_;
 
    $path         or throw Unspecified, [ 'path name' ], level => 2;
    blessed $path or throw 'Path [_1] is not blessed', [ $path ], level => 2;
-   $element      or throw 'Path [_1] result source not specified', [ $path ],
+   $rsrc_name    or throw 'Path [_1] result source not specified', [ $path ],
                           level => 2;
 
    return;
@@ -219,22 +246,22 @@ sub write_file {
    try {
       $create or $exists or throw PathNotFound, [ $path ];
       $exists or $path->perms( $self->_perms );
+      $self->atomic_write and $path->atomic;
 
       if ($exists and $self->backup and not $path->empty) {
          copy( "${path}", $path.$self->backup )
             or throw 'Backup copy failed: [_1]', [ $OS_ERROR ];
       }
 
-      try   { $data = $self->write_to_file( $path->atomic->lock, $data );
-              $path->close }
+      try   { $data = $self->write_to_file( $path->lock, $data ); $path->close }
       catch { $path->delete; throw $_ };
 
       $self->_cache->remove( $path );
       $self->_log->debug( "Write file ${path}" )
    }
-   catch { $self->_lock->reset( k => $path ); throw $_ };
+   catch { $self->$_lock_reset( $path ); throw $_ };
 
-   $self->_lock->reset( k => $path );
+   $self->$_lock_reset( $path );
    return $data;
 }
 
@@ -274,6 +301,11 @@ Defines the following attributes;
 
 =over 3
 
+=item C<atomic_write>
+
+Hash reference containing the keys of any locks set by the storage object.
+Used during object destruction to free any left over locks
+
 =item C<backup>
 
 Extension appended to the file name. Used to create a backup of the updated
@@ -289,10 +321,20 @@ to the null string
 The filename extension for this type of file. Usually overridden in the
 subclass. Default to the null string
 
+=item C<read_options>
+
+This hash reference is used to customise the decoder object used when
+reading the file. It defaults to an empty reference
+
 =item C<schema>
 
 A weakened schema object reference
 
+=item C<write_options>
+
+This hash reference is used to customise the encoder object used when
+writing the file. It defaults to an empty reference
+
 =back
 
 =head1 Subroutines/Methods
@@ -301,14 +343,21 @@ A weakened schema object reference
 
    $bool = $self->create_or_update( $path, $result, $updating, $condition );
 
-Does the heavy lifting for L</insert> and L</update>
+Does the heavy lifting for L</insert> and L</update>. The C<$updating> boolean
+is true for updating false otherwise. The C<$condition> code reference is
+used to filter updates
 
 =head2 delete
 
    $bool = $storage->delete( $path, $result );
 
-Deletes the specified element object returning true if successful. Throws
-an error otherwise. Path is an instance of L<File::DataClass::IO>
+Deletes the specified result object returning true if successful. Throws
+an error otherwise. Path is an instance of L<File::DataClass::IO>. The
+result is an instance of L<File::DataClass::Result>
+
+=head2 DEMOLISH
+
+Called during object destruction it deletes any outstanding locks
 
 =head2 dump
 
@@ -321,8 +370,9 @@ L<File::DataClass::IO>
 
    $bool = $storage->insert( $path, $result );
 
-Inserts the specified element object returning true if successful. Throws
-an error otherwise. Path is an instance of L<File::DataClass::IO>
+Inserts the specified result object returning true if successful. Throws
+an error otherwise. Path is an instance of L<File::DataClass::IO>. The
+result is an instance of L<File::DataClass::Result>
 
 =head2 load
 
@@ -333,8 +383,14 @@ it returns. Paths are instances of L<File::DataClass::IO>
 
 =head2 meta_pack
 
+Converts from scalar to hash reference. The scalar is the modification time
+of the file
+
 =head2 meta_unpack
 
+Converts from hash reference to scalar. The scalar is the modification time
+of the file
+
 =head2 read_file
 
    ($data, $mtime) = $self->read_file( $path, $for_update ):
@@ -351,8 +407,8 @@ Should be overridden in the subclass
 
    $hash_ref = $storage->select( $path );
 
-Returns a hash ref containing all the elements of the type specified in the
-schema. Path is an instance of L<File::DataClass::IO>
+Returns a hash reference containing all the records for the result source
+specified in the schema. Path is an instance of L<File::DataClass::IO>
 
 =head2 txn_do
 
@@ -362,14 +418,15 @@ Executes the supplied coderef wrapped in lock on the pathname
 
    $bool = $storage->update( $path, $result, $updating, $condition );
 
-Updates the specified element object returning true if successful. Throws
-an error otherwise. Path is an instance of L<File::DataClass::IO>
+Updates the specified result object returning true if successful. Throws
+an error otherwise. Path is an instance of L<File::DataClass::IO>. The
+result is an instance of L<File::DataClass::Result>
 
 =head2 validate_params
 
-   $storage->validate_params( $path, $element );
+   $storage->validate_params( $path, $rsrc_name );
 
-Throw if C<$path> or C<$element> are not specified or C<$path> is not blessed
+Throw if C<$path> or C<$rsrc_name> are not specified or C<$path> is not blessed
 
 =head2 write_file
 
@@ -392,7 +449,7 @@ None
 
 =over 3
 
-=item L<File::DataClass::HashMerge>
+=item L<Unexpected>
 
 =back
 
